/*global fonts, fbLike */

(function ($, _, Backbone, app) {
    app.views.inputView = Backbone.View.extend({

        events: {
            'focus': 'hide_placeholder',
            'blur': 'show_placeholder'
        },

        initialize: function () {
            _.bindAll(this, 'hide_placeholder', 'show_placeholder');
            this.placeholder = this.$el.attr('placeholder');
        },

        hide_placeholder: function () {
            this.$el.removeAttr('placeholder');
        },

        show_placeholder: function () {
            this.$el.attr('placeholder', this.placeholder);
        },

        val: function () {
            return this.$el.val();
        }
    });

    app.views.popupView = Backbone.View.extend({
        initialize: function () {

        },

        cleanTouchBehaviour: function () {
            $(document).off('touchmove');
            $('body').off('touchmove', '.scrollable');
            $('body').off('touchstart', '.scrollable');
            $('.mfp-container').removeClass('scrollable');
        },

        addTouchBehavior: function () {
            $('.mfp-container').addClass('scrollable');

            $(document).on('touchmove', function (e) {
                e.preventDefault();
            });

            $('body').on('touchstart', '.scrollable', function (e) {
                if (e.currentTarget.scrollTop === 0) {
                    e.currentTarget.scrollTop = 1;
                } else if (e.currentTarget.scrollHeight === e.currentTarget.scrollTop + e.currentTarget.offsetHeight) {
                    e.currentTarget.scrollTop -= 1;
                }
            });

            if ($('.mfp-content').height() + 100 > window.innerHeight) {
                $('body').on('touchmove', '.scrollable', function (e) {
                    e.stopPropagation();
                });
            }
        }

    });

    app.views.imagePopupView = app.views.popupView.extend({
        el: $('.popup-image'),

        initialize: function () {
            var _this = this;

            this.$el.magnificPopup({
                type: 'image',
                image: {
                    markup: '<div class="mfp-figure">' +
                    '<div class="mfp-close"></div>' +
                    '<div class="mfp-img"></div>' +
                    '<div class="mfp-bottom-bar">' +
                    '<div class="mfp-title"></div>' +
                    '</div>' +
                    '</div>',
                    titleSrc: function (item) {
                        return item.el.siblings('figcaption').html();
                    },
                    cursor: null,
                    verticalFit: false,
                    alignTop: false
                },
                callbacks: {
                    imageLoadComplete: function () {
                        _this.addTouchBehavior();
                    },

                    close: function () {
                        _this.cleanTouchBehaviour();
                    }
                }

            });
        }

    });

    app.views.galleryPopupView = app.views.popupView.extend({

        initialize: function () {
            var _this = this;

            this.$el.magnificPopup({
                delegate: 'figure > a',
                type: 'image',
                gallery: {
                    enabled: true,
                    navigateByImgClick: true,
                    preload: [0, 1] // Will preload 0 - before current, and 1 after the current image
                },
                image: {
                    markup: '<div class="mfp-figure">' +
                    '<div class="mfp-close"></div>' +
                    '<div class="mfp-counter"></div>' +
                    '<div class="mfp-img"></div>' +
                    '<div class="mfp-bottom-bar">' +
                    '<div class="mfp-title"></div>' +
                    '</div>' +
                    '</div>',
                    titleSrc: function (item) {
                        return item.el.siblings('figcaption').html();
                    },
                    cursor: null,
                    verticalFit: true
                },
                callbacks: {
                    imageLoadComplete: function () {
                        _this.addTouchBehavior();
                    },

                    change: function () {
                        _this.cleanTouchBehaviour();
                        _this.addTouchBehavior();
                    },

                    close: function () {
                        _this.cleanTouchBehaviour();
                    }
                }
            });
        }
    });

    app.views.galleryInlineView = Backbone.View.extend({
        initialize: function () {
            _.bindAll(this, 'renderCounter');
            var _this = this;
            this.$el.flexslider({
                controlNav: false,
                slideshow: false,
                selector: '.slides > figure',
                keyboard: false,
                start: function (slider) {
                    _this.$el.find('.total-slides').text(slider.count);
                    _this.$el.find('.current-slide').text(slider.currentSlide + 1);
                },
                before: function (slider) {
                    _this.$el.find('.current-slide').text(slider.animatingTo + 1);
                }
            });

            this.$el.find('.flex-direction-nav').append(this.renderCounter());
            fonts.done(function () {
                _this.centerArrows();
            });
        },

        renderCounter: function () {
            return '<li class="slide-counter"><span class="current-slide"></span>/<span class="total-slides">' +
                '</span></li>';
        },

        centerArrows: function () {
            var captionHeights = this.$el.find('figcaption').map(function (index, item) {
                return $(item).outerHeight(true);
            });
            var maxHeight = Math.max.apply(Math, captionHeights);
            this.$el.find('.flex-direction-nav a').css('top', 'calc((100% - ' + maxHeight + 'px) * 0.5)');
            this.$el.find('.slide-counter').css('top', 'calc((100% - ' + maxHeight + 'px) * 0.5)');
        }
    });

    app.views.footnoteView = Backbone.View.extend({
        el: $('.article-page'),

        events: {
            'click': 'checkIfClose',
            'click .footnote': 'openFootnote',
            'click .footnote-modal button': 'closeFootnote'
        },

        initialize: function () {
            this.$modal = undefined;
            this.invoker = undefined;
            this.invokerContents = undefined;
            this.contentWidth = $('.content').outerWidth();
            this.modalPadding = 20;
            this.openPhase = false;
            this.footnoteOpened = false;
        },

        openFootnote: function (e) {
            var _this = this;
            // Если повторно нажимаем по ссылке вызывающей сноску, то она закрывается
            if (!_.isUndefined(this.invoker) && this.invoker[0] === $(e.target)[0]) {
                this.closeFootnote();
                return;
            }

            // Если вызываем новую сноску, то предыдущую закрываем
            if (!_.isUndefined(this.$modal) && !_.isUndefined(this.invoker)) {
                this.closeFootnote();
            }

            this.invoker = $(e.target).closest('.footnote');
            this.invokerContents = $(e.target).html();
            this.openPhase = true;

            // Вставляем временныем маркеры для вычесления позиции текста внутри них
            this.invoker.prepend('<span class="open-marker"></span>');
            this.invoker.append('<span class="close-marker"></span>');

            // Вычисляем позицию и ширину элемента над которым необходимо отобразить сноску
            var positionParams = this.calculatePositionParams(e);

            var footnoteContents = $('.footnote-contents').filter(function () {
                return $(this).data("id") == _this.invoker.data('id')
            }).html();

            this.$modalElements = $('<div class="footnote-modal"><button data-action="close"></button>' + footnoteContents +
                '</div><div class="footnote-pointer-triangle"></div>');
            this.$modalElements.css({visibility: 'hidden'});
            $('article').append(this.$modalElements);
            this.$triangle = this.$el.find('.footnote-pointer-triangle');
            this.$modal = this.$el.find('.footnote-modal');

            var modalTop = positionParams.position.top - this.$modal.outerHeight() - 15;
            var modalLeft = positionParams.position.left + (positionParams.width / 2) - 89;

            this.viewportWidth = Math.max(document.documentElement.clientWidth, window.innerWidth || 0);
            var modalHorizontalWidth = modalLeft + this.$modal.outerWidth();
            var sideMargin = 0;
            if (this.viewportWidth - this.contentWidth >= 0) {
                sideMargin = this.viewportWidth - this.contentWidth;
            }
            if ((sideMargin + modalHorizontalWidth) >= (this.viewportWidth - 30)) {
                modalLeft -= (sideMargin + modalHorizontalWidth) - this.viewportWidth + 30;
            }

            modalLeft = Math.max(modalLeft, (positionParams.position.left + positionParams.width - this.$modal.outerWidth()));

            var triangleTop = positionParams.position.top - (this.$triangle.height() / 2 + 1);
            var triangleLeft = positionParams.position.left + (positionParams.width / 2) - (this.$triangle.width() / 2);

            //TODO: Сделать центрирование треугольника внизу окна
//            var shift = (modalLeft + this.$modal.outerWidth()) - (this.contentWidth - this.modalPadding);
//            if (shift > 0) {
//                modalLeft -= shift;
//            }

            this.$modal.css({position: 'absolute', top: modalTop, left: modalLeft, visibility: 'visible'});
            this.$triangle.css({position: 'absolute', top: triangleTop, left: triangleLeft, visibility: 'visible'});
            this.footnoteOpened = true;
        },

        calculatePositionParams: function (e) {
            var offset = this.invoker.offset();
            // позиция клика мышки внутри вызвавшего его элемента
            var clickY = e.pageY - offset.top;
            var invokerPosition = this.invoker.position();
            var $openMarker = $('.open-marker').position();
            var $closeMarker = $('.close-marker').position();

            if (this.isMultiline(e)) {
                var lineHeight = parseInt(this.invoker.closest('p').css('line-height'), 10);

                // Если клик попал на первую строку
                if (clickY >= 0 && clickY <= $openMarker.top + lineHeight) {
                    return {
                        position: {
                            top: invokerPosition.top + $openMarker.top,
                            left: invokerPosition.left + $openMarker.left
                        },
                        width: this.invoker.width() - $openMarker.left
                    }
                }
                // Если клик попал на последнюю строку
                else if (clickY >= $closeMarker.top && clickY <= $closeMarker.top + lineHeight) {
                    return {
                        position: {
                            top: invokerPosition.top + $closeMarker.top,
                            left: invokerPosition.left
                        },
                        width: $closeMarker.left
                    }
                }
                // Если клик попал на строку посередине
                else {
                    return {
                        position: {
                            top: invokerPosition.top + Math.floor(clickY / lineHeight) * lineHeight,
                            left: invokerPosition.left
                        },
                        width: this.invoker.width()
                    }
                }
            } else {
                return {
                    position: this.invoker.position(),
                    width: this.invoker.width()
                }
            }
        },

        isMultiline: function (e) {
            return $('.open-marker').position().top !== $('.close-marker').position().top;
        },

        closeFootnote: function () {
            this.$modal.remove();
            this.$triangle.remove();
            this.invoker.html(this.invokerContents);
            this.invokerContents = undefined;
            this.invoker = undefined;
            this.footnoteOpened = false;
        },

        checkIfClose: function (e) {
            if (this.openPhase) {
                this.openPhase = false;
                return;
            }
            if (!$(e.target).is('.footnote-modal') && !$(e.target).closest('.footnote-modal').length
                && this.footnoteOpened) {
                this.closeFootnote();
            }
        }
    });

    app.views.shareView = Backbone.View.extend({
        el: $('.article-footer'),

        events: {
            'click .facebook': 'showFacebook'
        },

        initialize: function () {
            this.setCount(settings.facebookAPIUrl, this.$el.find('.facebook + .counter'));
        },

        setCount: function (apiUrl, $element) {
            var requests = [];

            for (var i = 0; i < window.articleUrls.length; i++) {
                requests.push(this.getCount(apiUrl + encodeURIComponent(window.articleUrls[i])));
            }

            $.when.apply($, requests).done(function () {
                var count = 0;

                // TODO: Refactor into prettier sample
                if (arguments[1] === 'success') {
                    count += arguments[0].share.share_count;
                } else {
                    for (var i = 0; i < arguments.length; i++) {
                        count += arguments[i][0].share.share_count;
                    }
                }

                $element.text(count);
            });
        },

        getCount: function (url) {
            return $.ajax({
                dataType: "json",
                url: url,
                success: function (data) {

                }
            });
        },

        showFacebook: function () {
            window.open(
                'https://www.facebook.com/sharer/sharer.php?u=' + encodeURIComponent(location.href),
                'facebooksharedialog',
                'width=626,height=436');
            return false;
        }
    });

    app.views.facebookShareView = Backbone.View.extend({
        el: $('.facebook-block'),

        initialize: function () {
            var _this = this;
            fbLike.done(function () {
                _this.$el.find('.counter').addClass('hidden');
                _this.$el.find('.fb-container').addClass('hidden');
                _this.$el.find('.facebook-main').addClass('hidden');
                _this.$el.find('.message').removeClass('hidden');
            });
        },

        share: function (e) {
            var _this = this;
            e.preventDefault();
            FB.ui({
                method: 'feed',
                picture: 'http://thetype.astroshock.ru/static/img/content/fb_logo.jpg',
                name: 'Журнал «Шрифт» (Typejournal.ru)',
                link: 'http://typejournal.ru/',
                caption: 'typejournal.ru',
                description: 'Журнал «Шрифт» (Typejournal.ru) — это электронный журнал о шрифте и типографике, языке и времени, людях и визуальных традициях.'
            }, function (response) {
                if (response === null || response === undefined) {
                    console.log('Не удалось сделать facebook share');
                } else {
                    _this.$el.find('.counter').addClass('hidden');
                    _this.$el.find('.message').removeClass('hidden');
                }
            });
        }
    });

    app.views.simpleSubscribeView = Backbone.View.extend({
        el: $('.subscribe'),

        events: {
            'click form button': 'subscribe',
            'keypress form input': 'checkEnter'
        },

        initialize: function () {
            app.views.emailInput = new app.views.inputView({
                el: $('input.email')
            });

            var _this = this;
            this.$el.on('propertychange paste dragend dragover draggesture dragdrop drop drag keyup keydown keypress', 'form input', function () {
                _this.checkEmail();
            });
        },

        checkEmail: function () {
            var email = app.views.emailInput.val();
            var $button = this.$el.find('button');
            if (app.tools.validEmail(email)) {
                $button.addClass('active');
                $button.text('OK!');
            } else {
                $button.removeClass('active');
                $button.text('OK?');
            }
        },

        checkEnter: function (e) {
            if (e.keyCode === 13) {
                this.subscribe(e);
            }
        },

        subscribe: function (e) {
            e.preventDefault();

            var self = this;
            var email = app.views.emailInput.val();
            var lang = $('html').attr('lang');

            if (app.tools.validEmail(email)) {
                $.ajax({
                    url: settings.subscribeUrl,
                    type: "POST",
                    data: {
                        cmd: 'subscribe',
                        email: email,
                        lang: lang
                    },
                    success: function () {
                        self.$el.find('form').addClass('hidden');
                        self.$el.find('.message.success').removeClass('hidden');
                    },
                    error: function () {
                        self.$el.find('form').addClass('hidden');
                        self.$el.find('.message.failure').removeClass('hidden');
                    }
                });

            }
        }
    });

    /* View "О проекте" */
    app.views.aboutView = Backbone.View.extend({
        el: $('.about-page'),

        initialize: function () {
            app.views.simpleSubscribe = new app.views.simpleSubscribeView();
            app.views.facebookShare = new app.views.facebookShareView();
        }
    });

    /* View Статьи */
    app.views.articleView = Backbone.View.extend({
        el: $('.article-page'),

        initialize: function () {
            this.$el.find('article > p:last-of-type').append('<span class="icon-end"></span>');

            app.views.imagePopup = new app.views.imagePopupView();

            app.views.galleriesPopup = [];
            $('.popup-gallery').each(function (index, element) {
                app.views.galleriesPopup.push(new app.views.galleryPopupView({el: element}));
            });

            app.views.galleriesInline = [];

            $('.flexslider').each(function (index, element) {
                app.views.galleriesInline.push(new app.views.galleryInlineView({el: element}));
            });

            app.views.footnote = new app.views.footnoteView();

            app.views.simpleSubscribe = new app.views.simpleSubscribeView();
            app.views.facebookShare = new app.views.facebookShareView();
            app.views.share = new app.views.shareView();
        }
    });

    app.views.mainView = Backbone.View.extend({
        el: $('.main-page'),

        initialize: function () {
            var $container = $('.masonry ul');

            fonts.done(function () {
                $container.imagesLoaded(function () {

                    var msnry = new Masonry($container.get(0), {
                        columnWidth: 270,
                        itemSelector: '.article-item',
                        gutter: 60,
                        isAnimated: true
                    });

                    var $masonry = $('.masonry');
                    var fadeIteration = 0;

                    msnry.on('layoutComplete', function () {
                        // TODO: Починить с учетом случая когда опубликованных статей меньше 4,
                        // сейчас если меньше 4, то аннонсированные не показываются
                        function fadeArticleItem() {
                            if (fadeIteration < 4) {
                                $('.hidden-opacity').not('.disabled').eq(0).animate({opacity: 1}, 350, function () {
                                    $(this).removeClass('hidden-opacity');
                                    setTimeout(function () {
                                        fadeIteration += 1;
                                        fadeArticleItem();
                                    }, 120);
                                });
                            } else {
                                $('.hidden-opacity').not('.disabled').animate({opacity: 1}, 350, function () {
                                    $(this).removeClass('hidden-opacity');
                                });
                                $('.hidden-opacity.disabled').animate({opacity: 0.45}, 350, function () {
                                    $(this).removeClass('hidden-opacity');
                                });
                            }
                        }

                        setTimeout(function () {
                            fadeArticleItem();
                        }, 500);
                    });

                    msnry.layout();
                });
            });

            app.views.simpleSubscribe = new app.views.simpleSubscribeView();
            app.views.facebookShare = new app.views.facebookShareView();

        }
    });

    app.views.contentsView = Backbone.View.extend({
        el: $('.contents-page'),

        initialize: function () {
            $('#filter-select').selectBox({
                mobile: true,
                menuTransition: 'slide',
                menuSpeed: 'normal'
            });

            $(".toggle_plus").click(function () {
                var parent = $(this).parents(".tr-bottom-line");
                var clicked = $(this);
                var layer = parent.find(".layer_toggle");
                layer.slideToggle(
                    function () {
                        if (clicked.hasClass("min"))
                            clicked.removeClass("min").addClass("plus")
                        else
                            clicked.removeClass("plus").addClass("min")
                    })
            });

            app.views.simpleSubscribe = new app.views.simpleSubscribeView();
            app.views.facebookShare = new app.views.facebookShareView();
        }
    });
})(jQuery, _, Backbone, application);
